# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'bubble/bclient/ui/wdark.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WDark(object):
    def setupUi(self, WDark):
        WDark.setObjectName("WDark")
        WDark.resize(759, 348)
        WDark.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout = QtWidgets.QVBoxLayout(WDark)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(WDark)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.bkgLineEdit = QtWidgets.QLineEdit(WDark)
        self.bkgLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.bkgLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.bkgLineEdit.setText("")
        self.bkgLineEdit.setObjectName("bkgLineEdit")
        self.gridLayout.addWidget(self.bkgLineEdit, 0, 1, 1, 1)
        self.bkgButton = QtWidgets.QPushButton(WDark)
        self.bkgButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/folder"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bkgButton.setIcon(icon)
        self.bkgButton.setObjectName("bkgButton")
        self.gridLayout.addWidget(self.bkgButton, 0, 2, 1, 1)
        self.bkgClearButton = QtWidgets.QPushButton(WDark)
        self.bkgClearButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/clear"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bkgClearButton.setIcon(icon1)
        self.bkgClearButton.setObjectName("bkgClearButton")
        self.gridLayout.addWidget(self.bkgClearButton, 0, 3, 1, 1)
        self.label_2 = QtWidgets.QLabel(WDark)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.sampleLineEdit = QtWidgets.QLineEdit(WDark)
        self.sampleLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.sampleLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.sampleLineEdit.setText("")
        self.sampleLineEdit.setObjectName("sampleLineEdit")
        self.gridLayout.addWidget(self.sampleLineEdit, 1, 1, 1, 1)
        self.sampleButton = QtWidgets.QPushButton(WDark)
        self.sampleButton.setText("")
        self.sampleButton.setIcon(icon)
        self.sampleButton.setObjectName("sampleButton")
        self.gridLayout.addWidget(self.sampleButton, 1, 2, 1, 1)
        self.sampleClearButton = QtWidgets.QPushButton(WDark)
        self.sampleClearButton.setText("")
        self.sampleClearButton.setIcon(icon1)
        self.sampleClearButton.setObjectName("sampleClearButton")
        self.gridLayout.addWidget(self.sampleClearButton, 1, 3, 1, 1)
        self.label_3 = QtWidgets.QLabel(WDark)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.splineLineEdit = QtWidgets.QLineEdit(WDark)
        self.splineLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.splineLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.splineLineEdit.setText("")
        self.splineLineEdit.setObjectName("splineLineEdit")
        self.gridLayout.addWidget(self.splineLineEdit, 2, 1, 1, 1)
        self.splineButton = QtWidgets.QPushButton(WDark)
        self.splineButton.setText("")
        self.splineButton.setIcon(icon)
        self.splineButton.setObjectName("splineButton")
        self.gridLayout.addWidget(self.splineButton, 2, 2, 1, 1)
        self.splineClearButton = QtWidgets.QPushButton(WDark)
        self.splineClearButton.setText("")
        self.splineClearButton.setIcon(icon1)
        self.splineClearButton.setObjectName("splineClearButton")
        self.gridLayout.addWidget(self.splineClearButton, 2, 3, 1, 1)
        self.label_4 = QtWidgets.QLabel(WDark)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.floodLineEdit = QtWidgets.QLineEdit(WDark)
        self.floodLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.floodLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.floodLineEdit.setText("")
        self.floodLineEdit.setObjectName("floodLineEdit")
        self.gridLayout.addWidget(self.floodLineEdit, 3, 1, 1, 1)
        self.floodButton = QtWidgets.QPushButton(WDark)
        self.floodButton.setText("")
        self.floodButton.setIcon(icon)
        self.floodButton.setObjectName("floodButton")
        self.gridLayout.addWidget(self.floodButton, 3, 2, 1, 1)
        self.floodClearButton = QtWidgets.QPushButton(WDark)
        self.floodClearButton.setText("")
        self.floodClearButton.setIcon(icon1)
        self.floodClearButton.setObjectName("floodClearButton")
        self.gridLayout.addWidget(self.floodClearButton, 3, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.incidenceCheckBox = QtWidgets.QCheckBox(WDark)
        self.incidenceCheckBox.setObjectName("incidenceCheckBox")
        self.verticalLayout.addWidget(self.incidenceCheckBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(WDark)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(WDark)
        self.buttonBox.accepted.connect(WDark.accept)
        self.buttonBox.rejected.connect(WDark.reject)
        QtCore.QMetaObject.connectSlotsByName(WDark)

    def retranslateUi(self, WDark):
        _translate = QtCore.QCoreApplication.translate
        WDark.setWindowTitle(_translate("WDark", "Frelon corrections"))
        self.label.setText(_translate("WDark", "Dark background"))
        self.label_2.setText(_translate("WDark", "Dark sample"))
        self.label_3.setText(_translate("WDark", "Spline"))
        self.label_4.setText(_translate("WDark", "Flood"))
        self.incidenceCheckBox.setText(_translate("WDark", "Incidence angle correction (good for PDF)"))

from . import resources_rc
